/* $Id: diskdumpcmn.c,v 1.2 2005/02/25 20:59:44 akira Exp $ */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "version.h"
#include "diskdumpcmn.h"

void version(const char *cmd)
{
	fprintf(stderr, "%s version %s\n", cmd, CMD_VERSION);

	exit(0);
}

int convert_version_number(struct version_form *ver, const char *v_str)
{
	/* Initialize the version number as "0.0.0". */
	ver->major = 0;
	ver->minor = 0;
	ver->micro = 0;

	return sscanf(v_str, "%d.%d.%d", &ver->major, &ver->minor, &ver->micro);
}

/*
 * It returns not actual diskdump module path but module path ID.
 */
int find_module_path(void)
{
	FILE *fp;
	struct stat stt;
	char buf[BUFSIZ];

	if ((fp = fopen("/etc/mtab", "r")) == NULL) {
		return INVALID_MODULE_ID;
	}

	while (fgets(buf, BUFSIZ, fp) != NULL) {
		if (strstr(buf, " /sys sysfs ") != NULL) {
			/* kernel-2.6.x */
			if (stat("/sys/module/diskdump/parameters", &stt)
			== 0) {
				/* kernel-2.6.10 */
				fclose(fp);
				return SYS_MODULE_PARAMETERS;
			} else {
				fclose(fp);
				return SYS_MODULE_DISKDUMP;
			}
		} else if (strstr(buf, " /proc ") != NULL) {
			/* kernel-2.4.x */
			fclose(fp);
			return PROC_DISKDUMP;
		}
	}

	fclose(fp);

	return INVALID_MODULE_ID;
}

/*
 * It gets some format parameters, sample_rate, block_order,
 * total_blocks and so on, from "/proc/diskdump".
 * This function is for kernel-2.4.x.
 */
u64 scan_param_proc(const char *param_name)
{
	FILE *fp;
	char buf[BUFSIZ];
	u64 val;
	const u64 failed = (u64)-1;

	if ((fp = fopen("/proc/diskdump", "r")) == NULL) {
		return failed;
	}

	while (fgets(buf, BUFSIZ, fp) != NULL) {
		if (strstr(buf, param_name) != NULL) {
			if (sscanf(buf, "# %*s %Lu", &val) != 1) {
				fclose(fp);
				return failed;
			}
			fclose(fp);
			return val;	/* Got an exact value. */
		}
	}

	return failed;
}

/*
 * It gets some format parameters, sample_rate, block_order,
 * total_blocks and so on, from "/sys/module/diskdump/"
 * or "/sys/module/diskdump/parameters/".
 */
u64 scan_param_sys(int module_path, const char *param_name)
{
	FILE *fp;
	char *pre = "/sys/module/diskdump";
	char *file_name;
	u64 val;
	const u64 failed = (u64)-1;

	switch (module_path) {
	case SYS_MODULE_DISKDUMP:
		asprintf(&file_name, "%s/%s", pre, param_name);
		break;
	case SYS_MODULE_PARAMETERS:	/* for kernel-2.6.10 */
		asprintf(&file_name, "%s/parameters/%s", pre, param_name);
		break;
	default:
		return failed;
	}

	if ((fp = fopen(file_name, "r")) == NULL) {
		return failed;
	}

	if (fscanf(fp, "%Lu", &val) != 1) {
		fclose(fp);
		return failed;
	}

	fclose(fp);

	return val;	/* Got an exact value. */
}
