/*
 *  diskdumpctl_proc.c
 *
 *  Copyright (C) 2004, 2005  FUJITSU LIMITED
 *  Written by Tatsuo Uchida (tuchida@us.fujitsu.com)
 *
 *  Some codes were derived from netdump and copyright belongs to
 *  Red Hat, Inc.
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/* $Id: diskdumpctl_proc.c,v 1.5 2005/02/25 16:39:51 akira Exp $ */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include "version.h"
#include "diskdumpcmn.h"

#define CMD	"diskdumpctl"

#define BLKADDDUMPDEVICE	0xdf00
#define BLKREMOVEDUMPDEVICE	0xdf01

static void usage(const char *cmd)
{
	fprintf(stderr, "Usage: %s [-u] device\n", cmd);
	fprintf(stderr, "       %s -V\n", cmd);
        fprintf(stderr, "\n");
        fprintf(stderr, "Available options:\n");
        fprintf(stderr, "\t-u:   unregister the device\n");
        fprintf(stderr, "\t-V:   show %s version and exit\n", cmd);

	exit(1);
}

int main(int argc, char *argv[])
{
	int rc, dev_fd, proc_fd;
	int opt;
	int do_unregister = 0;

	while ((opt = getopt(argc, argv, "uV")) != -1) {
		switch (opt) {
		case 'u':
			do_unregister = 1;
			break;
		case 'V':
			version(CMD);
			break;
		default:
			usage(CMD);
			break;
		}
	}

	if (optind +1 != argc)
		usage(CMD);

	dev_fd = open(argv[optind], O_RDONLY);
	if (dev_fd < 0) {
		Syserr("cannot open device");
		exit(1);
	}
	proc_fd = open("/proc/diskdump", O_RDONLY);
	if (proc_fd < 0) {
		Syserr("cannot open /proc/diskdump");
		exit(1);
	}

	if (!do_unregister) {
		rc = ioctl(proc_fd, BLKADDDUMPDEVICE, dev_fd);
		if (rc < 0) {
			Syserr("cannot add dumpdevice");
			exit(1);
		}
	} else {
		rc = ioctl(proc_fd, BLKREMOVEDUMPDEVICE, dev_fd);
		if (rc < 0) {
			Syserr("cannot remove dumpdevice");
			exit(1);
		}
	}

	exit(0);
}
